import asyncio
import os
import uuid
from typing import Any, Dict
from unittest.mock import ANY, MagicMock, patch

import pytest

try:
    import langchain.chains
except ImportError:
    pytest.skip("langchain not installed", allow_module_level=True)
from langchain.chains import LLMChain
from langchain.llms import OpenAI
from langchain.load.dump import dumpd
from langchain.prompts import (
    AIMessagePromptTemplate,
    ChatPromptTemplate,
    HumanMessagePromptTemplate,
    MessagesPlaceholder,
    PromptTemplate,
    StringPromptTemplate,
    SystemMessagePromptTemplate,
)
from langchain.schema import (
    AIMessage,
    Generation,
    HumanMessage,
    LLMResult,
    SystemMessage,
)

from im_openai.langchain import callbacks as langchain_callbacks
from im_openai.langchain import util as langchain_util

api_key = "abc"
api_name = "def"


@pytest.fixture(autouse=True)
def openai_api_key():
    os.environ["OPENAI_API_KEY"] = "abc"
    yield
    del os.environ["OPENAI_API_KEY"]


@pytest.fixture()
def mock_send_event():
    with patch("im_openai.client.send_event", autospec=True) as mock:
        yield mock


@pytest.fixture()
def mock_event_loop():
    loop = asyncio.new_event_loop()
    asyncio.set_event_loop(loop)
    yield loop
    loop.close()


@pytest.fixture()
def pwc():
    return langchain_callbacks.PromptWatchCallbacks(api_key=api_key, api_name=api_name)


def test_llm_start(pwc: langchain_callbacks.PromptWatchCallbacks, mock_send_event: MagicMock):
    run_id = uuid.uuid4()
    parent_run_id = uuid.uuid4()
    prompt = PromptTemplate.from_template("What is a good name for a company that makes {product}?")
    chain = LLMChain(
        llm=OpenAI(client=None, model="text-davinci-003", temperature=0.5),
        prompt=prompt,
    )
    run_llm_start(pwc, mock_send_event, run_id, parent_run_id, chain)

    mock_send_event.assert_called_once_with(
        ANY,
        api_key=api_key,
        api_name=api_name,
        project_key=None,
        prompt_template_text="What is a good name for a company that makes {product}?",
        prompt_template_chat=None,
        prompt_params={"product": "socks"},
        prompt_event_id=None,
        chat_id=None,
        model_params={"modelType": "completion", "temperature": 0.5, "model": "text-davinci-003"},
        response=None,
        response_time=None,
        prompt=None,
        parent_event_id=str(parent_run_id),
    )


def test_chat_model_start(
    pwc: langchain_callbacks.PromptWatchCallbacks, mock_send_event: MagicMock
):
    run_id = uuid.uuid4()
    parent_run_id = uuid.uuid4()
    template_str = (
        "You are a helpful assistant that translates {input_language} to {output_language}."
    )
    system_message_prompt = SystemMessagePromptTemplate.from_template(template_str)
    human_template = "{text}"
    human_message_prompt = HumanMessagePromptTemplate.from_template(human_template)

    template = ChatPromptTemplate.from_messages([system_message_prompt, human_message_prompt])
    chain = LLMChain(
        llm=OpenAI(client=None, model="text-davinci-003", temperature=0.5),
        prompt=template,
    )
    template_args = {
        "input_language": "English",
        "output_language": "French",
        "text": "Hello",
    }
    run_chat_model_start(
        pwc,
        mock_send_event,
        run_id,
        parent_run_id,
        chain,
        template_args,
    )

    mock_send_event.assert_called_once_with(
        ANY,
        api_key=api_key,
        api_name=api_name,
        project_key=None,
        prompt_template_text=None,
        prompt_template_chat=[
            {
                "content": "You are a helpful assistant that translates {input_language} to {output_language}.",
                "role": "system",
            },
            {"content": "{text}", "role": "user"},
        ],
        prompt_params=template_args,
        prompt_event_id=None,
        chat_id=None,
        response=None,
        response_time=None,
        prompt=None,
        model_params={"modelType": "chat", "temperature": 0.5, "model": "text-davinci-003"},
        parent_event_id=str(parent_run_id),
    )


def test_chat_model_template_no_vars(
    pwc: langchain_callbacks.PromptWatchCallbacks, mock_send_event: MagicMock
):
    run_id = uuid.uuid4()
    parent_run_id = uuid.uuid4()
    template_str = "You are a helpful assistant"
    system_message = SystemMessage(content=template_str)
    human_message = HumanMessage(content="Hello")
    template = ChatPromptTemplate.from_messages([system_message, human_message])
    chain = LLMChain(
        llm=OpenAI(client=None, model="text-davinci-003", temperature=0.5),
        prompt=template,
    )
    template_args = {}
    run_chat_model_start(
        pwc,
        mock_send_event,
        run_id,
        parent_run_id,
        chain,
        template_args,
    )

    mock_send_event.assert_called_once_with(
        ANY,
        api_key=api_key,
        api_name=api_name,
        project_key=None,
        prompt_template_text=None,
        prompt_template_chat=None,
        prompt_params=template_args,
        prompt_event_id=None,
        chat_id=None,
        response=None,
        response_time=None,
        # Comes in via the prompt rather than the prompt_template_chat/etc
        prompt={
            "chat": [
                {
                    "content": "You are a helpful assistant",
                    "role": "system",
                },
                {"content": "Hello", "role": "user"},
            ]
        },
        model_params={"modelType": "chat", "temperature": 0.5, "model": "text-davinci-003"},
        parent_event_id=str(parent_run_id),
    )


def test_chat_model_parent(
    pwc: langchain_callbacks.PromptWatchCallbacks,
    mock_send_event: MagicMock,
    mock_event_loop,
):
    new_event_id = uuid.uuid4()
    # new_event_future_id = asyncio.futures.Future()
    # new_event_future_id.set_result(new_event_id)
    mock_send_event.return_value = str(new_event_id)
    run_id = uuid.uuid4()
    parent_run_id = uuid.uuid4()
    template_str = (
        "You are a helpful assistant that translates {input_language} to {output_language}."
    )
    system_message_prompt = SystemMessagePromptTemplate.from_template(template_str)

    template = ChatPromptTemplate.from_messages([system_message_prompt])
    chain = LLMChain(
        llm=OpenAI(client=None, model="text-davinci-003", temperature=0.5),
        prompt=template,
    )
    template_args = {
        "input_language": "English",
        "output_language": "French",
    }
    run_chat_model_start(
        pwc,
        mock_send_event,
        run_id,
        parent_run_id,
        chain,
        template_args,
    )

    mock_send_event.assert_called_once_with(
        ANY,
        api_key=api_key,
        api_name=api_name,
        project_key=None,
        prompt_template_text=None,
        prompt_template_chat=[
            {
                "role": "system",
                "content": "You are a helpful assistant that translates {input_language} to {output_language}.",
            }
        ],
        prompt_params=template_args,
        prompt_event_id=None,
        chat_id=None,
        response=None,
        response_time=None,
        model_params={"modelType": "chat", "temperature": 0.5, "model": "text-davinci-003"},
        prompt=None,
        parent_event_id=str(parent_run_id),
    )
    mock_send_event.reset_mock()

    pwc.on_llm_end(
        LLMResult(generations=[[Generation(text="hi")]]),
        run_id=run_id,
        parent_run_id=parent_run_id,
    )

    mock_send_event.assert_called_once_with(
        ANY,
        api_key=api_key,
        api_name=api_name,
        project_key=None,
        prompt_template_text=None,
        prompt_template_chat=[
            {
                "role": "system",
                "content": "You are a helpful assistant that translates {input_language} to {output_language}.",
            }
        ],
        prompt_params=template_args,
        prompt_event_id=str(new_event_id),
        model_params={"modelType": "chat", "temperature": 0.5, "model": "text-davinci-003"},
        chat_id=None,
        response="hi",
        response_time=ANY,
        prompt=None,
        parent_event_id=str(parent_run_id),
    )


def run_llm_start(
    pwc: langchain_callbacks.PromptWatchCallbacks,
    mock_send_event: MagicMock,
    run_id: uuid.UUID,
    parent_run_id: uuid.UUID,
    chain: LLMChain,
):
    pwc.on_chain_start(
        dumpd(chain),
        dict(product="socks"),
        run_id=run_id,
        parent_run_id=parent_run_id,
    )

    mock_send_event.assert_not_called()

    pwc.on_llm_start(
        dumpd(chain.llm),
        run_id=run_id,
        parent_run_id=parent_run_id,
        prompts=[chain.prompt.format(product="socks")],
    )


def run_chat_model_start(
    pwc: langchain_callbacks.PromptWatchCallbacks,
    mock_send_event: MagicMock,
    run_id: uuid.UUID,
    parent_run_id: uuid.UUID,
    chain: LLMChain,
    args: Dict[str, Any],
):
    pwc.on_chain_start(
        dumpd(chain),
        args,
        run_id=run_id,
        parent_run_id=parent_run_id,
    )
    assert isinstance(chain.prompt, ChatPromptTemplate)

    mock_send_event.assert_not_called()

    messages = chain.prompt.format_messages(**args)
    # TODO: the type coercion here is a bit hacky
    n = langchain_util.format_chat_template(messages)  # type: ignore
    pwc.on_chat_model_start(
        dumpd(chain.llm),
        messages=[n],  # type: ignore
        run_id=run_id,
        parent_run_id=parent_run_id,
        tags=None,
        metadata=None,
    )


def test_format_completion_with_inputs():
    template = ChatPromptTemplate.from_messages([])
    s = langchain_util.format_completion_template_with_inputs(template, {})
    assert s == ""

    template = ChatPromptTemplate.from_messages([HumanMessagePromptTemplate.from_template("{a}")])
    s = langchain_util.format_completion_template_with_inputs(template, {"a": "b"})
    assert s == "Human: {a}"

    template = ChatPromptTemplate.from_messages([AIMessagePromptTemplate.from_template("{a}")])
    s = langchain_util.format_completion_template_with_inputs(template, {"a": "b"})
    assert s == "AI: {a}"

    template = ChatPromptTemplate.from_messages([SystemMessagePromptTemplate.from_template("{a}")])
    s = langchain_util.format_completion_template_with_inputs(template, {"a": "b"})
    assert s == "System: {a}"

    # Multiple
    template = ChatPromptTemplate.from_messages(
        [
            SystemMessagePromptTemplate.from_template("{a}"),
            HumanMessagePromptTemplate.from_template("{b}"),
            AIMessagePromptTemplate.from_template("{c}"),
        ]
    )
    s = langchain_util.format_completion_template_with_inputs(
        template, {"a": "x", "b": "y", "c": "z"}
    )
    assert (
        s
        == """System: {a}
Human: {b}
AI: {c}"""
    )


def test_format_chat_template_with_inputs_zero():
    template = ChatPromptTemplate.from_messages([])
    msgs = langchain_util.format_chat_template_with_inputs(template, {})
    assert msgs == []


def test_format_chat_template_with_inputs_one():
    template = ChatPromptTemplate.from_messages([HumanMessagePromptTemplate.from_template("{a}")])
    msgs = langchain_util.format_chat_template_with_inputs(template, {"a": "b"})
    assert msgs == [{"role": "user", "content": "{a}"}]

    # Empty chat history


def test_format_chat_template_with_inputs_chat_history_zero():
    template = ChatPromptTemplate.from_messages([MessagesPlaceholder(variable_name="a")])
    msgs = langchain_util.format_chat_template_with_inputs(template, {"a": []})
    # TODO: should be {"role": "chat_history", "content": "{a}"}
    assert msgs == []


def test_format_chat_template_with_inputs_chat_history_one():
    # One item, still emits one item
    template = ChatPromptTemplate.from_messages([MessagesPlaceholder(variable_name="a")])
    inputs = {"a": [HumanMessage(content="Hi")]}
    msgs = langchain_util.format_chat_template_with_inputs(template, inputs)
    msgs = langchain_util.replace_array_variables_with_placeholders(msgs, inputs)
    assert msgs == [{"role": "chat_history", "content": "{a}"}]


def test_format_chat_template_with_inputs_chat_history_two():
    # Two items, still emits one item for the placeholder
    template = ChatPromptTemplate.from_messages([MessagesPlaceholder(variable_name="a")])
    inputs = {"a": [HumanMessage(content="Hi"), AIMessage(content="Welcome.")]}
    msgs = langchain_util.format_chat_template_with_inputs(template, inputs)
    msgs = langchain_util.replace_array_variables_with_placeholders(msgs, inputs)
    assert msgs == [{"role": "chat_history", "content": "{a}"}]


def test_replace_array_variables_with_placeholders_none():
    msgs = [{"role": "system", "content": "Basic content"}]
    assert langchain_util.replace_array_variables_with_placeholders(msgs, {}) == msgs


def test_replace_array_variables_with_placeholders_nomatch():
    msgs = [{"role": "system", "content": "{a[0].role}"}]
    assert langchain_util.replace_array_variables_with_placeholders(msgs, {"a": []}) == msgs


def test_replace_array_variables_with_placeholders_match():
    msgs = [{"role": "{agent_history[0].role}", "content": "{agent_history[0].content}"}]
    assert langchain_util.replace_array_variables_with_placeholders(
        msgs, {"agent_history": [{"role": "agent", "content": "This is what an agent says"}]}
    ) == [{"role": "chat_history", "content": "{agent_history}"}]



from .callbacks import (
    PromptWatchCallbacks,
    disable_prompt_watch_tracing,
    enable_prompt_watch_tracing,
    prompt_watch_tracing,
)
from .patch import loads
from .util import format_chat_template



import uuid
from unittest.mock import ANY, MagicMock, patch

import openai
import pytest

from im_openai.patch import patch_openai


@pytest.fixture()
def mock_send_event():
    with patch("im_openai.client.send_event", autospec=True) as mock:
        yield mock


@pytest.fixture()
def do_patch_openai():
    unpatch = patch_openai()
    yield
    unpatch()


@pytest.fixture()
def mock_chat():
    with patch("openai.ChatCompletion.create", autospec=True) as mock:
        yield mock


def test_chat_completion(mock_chat, mock_send_event: MagicMock, do_patch_openai):
    template = "Send a greeting to our new user named {name}"
    ip_template_params = {"name": "Alec"}
    prompt_text = template.format(**ip_template_params)

    chat_messages = [{"role": "user", "content": prompt_text}]
    chat_template = [{"role": "user", "content": template}]
    api_key = "alecf-local-playground"
    api_name = "test-from-apitest-chat"
    event_id = uuid.uuid4()
    parent_event_id = uuid.uuid4()
    chat_id = uuid.uuid4()
    openai.ChatCompletion.create(
        model="gpt-3.5-turbo",
        messages=chat_messages,
        temperature=0.4,
        ip_api_key=api_key,
        ip_api_name=api_name,
        ip_template_chat=chat_template,
        ip_template_params=ip_template_params,
        ip_parent_event_id=parent_event_id,
        ip_event_id=event_id,
        ip_chat_id=chat_id,
    )

    assert tuple(mock_send_event.call_args_list[0]) == (
        (),
        dict(
            session=ANY,
            api_key=api_key,
            api_name=api_name,
            project_key=None,
            prompt_event_id=event_id,
            prompt_template_text=None,
            prompt_template_chat=[
                {"role": "user", "content": "Send a greeting to our new user named {name}"}
            ],
            prompt_params=ip_template_params,
            chat_id=chat_id,
            parent_event_id=parent_event_id,
            model_params={
                "model": "gpt-3.5-turbo",
                "modelProvider": "openai",
                "modelType": "chat",
                "temperature": 0.4,
            },
        ),
    )
    assert tuple(mock_send_event.call_args_list[1]) == (
        (),
        dict(
            session=ANY,
            api_key=api_key,
            api_name="test-from-apitest-chat",
            project_key=None,
            prompt_event_id=event_id,
            prompt_template_text=None,
            prompt_template_chat=[
                {"role": "user", "content": "Send a greeting to our new user named {name}"}
            ],
            prompt_params=ip_template_params,
            chat_id=chat_id,
            parent_event_id=parent_event_id,
            model_params={
                "model": "gpt-3.5-turbo",
                "modelProvider": "openai",
                "modelType": "chat",
                "temperature": 0.4,
            },
            response=ANY,
            response_time=ANY,
        ),
    )



from typing import Dict
from uuid import UUID


def find_leaves(parents: Dict[UUID, UUID]) -> set[UUID]:
    return set(child for child in parents.keys() if child not in parents.values())


def dump_node(parents: Dict[UUID, UUID], node: UUID, suffix: str = ""):
    if node not in parents:
        print(f"{node}{suffix}")
    else:
        parent = parents[node]
        dump_node(parents, parent, f" -> {node}{suffix}")


def dump_tree(parents: Dict[UUID, UUID]):
    for leaf in find_leaves(parents):
        dump_node(parents, leaf)



"""Utilities for sending events to Imaginary Dev."""
import asyncio
import logging
import os
import time
import uuid
import warnings
from contextlib import asynccontextmanager
from typing import Any, Dict, List, TypedDict

import aiohttp

logger = logging.getLogger(__name__)


async def send_event(
    session: aiohttp.ClientSession,
    api_key: str | None,
    *,
    project_key: str | None = None,
    api_name: str | None,
    prompt_event_id: str | None = None,
    prompt_template_text: str | None = None,
    prompt_template_chat: List | None = None,
    prompt_params: Dict | None = None,
    chat_id: str | None = None,
    response: str | None = None,
    response_time: float | None = None,
    prompt: Any | None = None,
    parent_event_id: str | None = None,
    model_params: Dict | None = None,
) -> str | None:
    """Send an event to Imaginary Dev. Returns the id of the event that was added on the server."""
    PROMPT_REPORTING_URL = os.environ.get(
        "PROMPT_REPORTING_URL", "https://app.imaginary.dev/api/event"
    )
    event = {
        "apiName": api_name,
        "params": {},
        "prompt": prompt or {},
        "promptEventId": prompt_event_id,
        "modelParameters": model_params or {},
    }

    logger.debug(f"Sending event to {PROMPT_REPORTING_URL}")
    if project_key is not None:
        event["projectKey"] = project_key
    if api_key is not None:
        event["apiKey"] = api_key

    if not api_key and not project_key:
        logger.warn("No project key or api key set, not sending event")
        return

    if prompt_template_text is not None:
        if isinstance(prompt_template_text, str):
            # first default template to just the raw text
            event["promptTemplateText"] = prompt_template_text
            if prompt_params is None and getattr(prompt_template_text, "params", None):
                # Can be TemplateString or any other
                prompt_params = prompt_template_text.params  # type: ignore

            # If the original template is available, send it too
            if getattr(prompt_template_text, "template", None):
                event["promptTemplateText"] = prompt_template_text.template  # type: ignore

    elif prompt_template_chat is not None:
        # We're going to assume that if a non-string was passed in, then it
        # was probably a chat template, aka a chat message history

        # Do any langchain conversion, in case there are langchain objects inside
        try:
            # import will fail if langchain is not installed
            from .langchain import format_chat_template

            prompt_template_chat = format_chat_template(prompt_template_chat)
        except ImportError:
            pass
        # TODO: figure out template extraction for chat templates
        event["promptTemplateChat"] = prompt_template_chat

    if prompt_params is not None:
        event["params"] = prompt_params
    if response is not None:
        event["response"] = response
    if response_time is not None:
        event["responseTime"] = response_time
    if chat_id is not None:
        event["chatId"] = chat_id
    if parent_event_id is not None:
        event["parentEventId"] = str(parent_event_id)

    result = await session.post(PROMPT_REPORTING_URL, json=event)
    json = await result.json()
    if isinstance(json, dict):
        return json.get("id")


@asynccontextmanager
async def event_session(
    project_key: str | None,
    api_key: str | None,
    api_name: str | None,
    prompt_template_text: str | None,
    prompt_template_chat: List | None,
    model_params: Dict | None = None,
    chat_id: str | None = None,
    prompt_template_params: dict | None = None,
    prompt_event_id: str | None = None,
    parent_event_id: str | None = None,
):
    """Context manager for sending an event to Imaginary Dev.

    Usage::

        with event_session(project_key, api_name, prompt_text, prompt_event_id) as complete_event:
            # do something
            complete_event(response)

    """
    start = time.time()
    if prompt_event_id is None:
        prompt_event_id = str(uuid.uuid4())

    async with aiohttp.ClientSession() as session:
        pending_events_sent = []
        first_event_sent = send_event(
            session=session,
            project_key=project_key,
            api_key=api_key,
            api_name=api_name,
            prompt_event_id=prompt_event_id,
            prompt_template_text=prompt_template_text,
            prompt_template_chat=prompt_template_chat,
            prompt_params=prompt_template_params,
            chat_id=chat_id,
            parent_event_id=parent_event_id,
            model_params=model_params,
        )
        pending_events_sent.append(first_event_sent)

        async def complete_event(response):
            response_time = (time.time() - start) * 1000
            second_event_sent = send_event(
                session=session,
                project_key=project_key,
                api_key=api_key,
                api_name=api_name,
                prompt_event_id=prompt_event_id,
                prompt_template_text=prompt_template_text,
                prompt_template_chat=prompt_template_chat,
                prompt_params=prompt_template_params,
                chat_id=chat_id,
                parent_event_id=parent_event_id,
                model_params=model_params,
                response=response,
                response_time=response_time,
            )
            pending_events_sent.append(second_event_sent)

        yield complete_event
        w = time.time()
        pending_events_results = await asyncio.gather(*pending_events_sent, return_exceptions=True)
        failed_events = [e for e in pending_events_results if isinstance(e, Exception)]
        if failed_events:
            warnings.warn(
                f"Failed to report calls. {len(failed_events)} failure(s). First failure: {failed_events[0]}",
                stacklevel=3,
            )



import asyncio
import os
from unittest.mock import AsyncMock, patch

import aiohttp
import pytest

from im_openai.client import send_event


@pytest.fixture
def mock_env(monkeypatch):
    monkeypatch.setitem(os.environ, "PROMPT_REPORTING_URL", "https://app.imaginary.dev/api/event")


@pytest.mark.asyncio
@patch("aiohttp.ClientSession.post", new_callable=AsyncMock)
@patch("os.environ.get")
async def test_send_event_chat(mock_env_get, mock_post, mock_env):
    # Mocking the environment variable for URL
    mock_env_get.return_value = "https://app.imaginary.dev/api/event"

    # Mocking the response from the aiohttp post call
    mock_response = AsyncMock()
    mock_response.json.return_value = {"id": "test_event_id"}
    mock_post.return_value = mock_response

    # Creating a ClientSession object
    async with aiohttp.ClientSession() as session:
        # Testing the send_event function with required parameters
        api_key = "test_api_key"
        prompt_template_chat = [{"role": "user", "content": "Hello world"}]
        prompt_params = {"param1": "value1", "param2": "value2"}
        chat_id = "test_chat_id"
        result = await send_event(
            session,
            api_key,
            api_name="test_api_name",
            prompt_event_id="test_prompt_event_id",
        )
        prompt_params = {"param1": "value1", "param2": "value2"}
        chat_id = "test_chat_id"
        result = await send_event(
            session,
            api_key,
            api_name="test_api_name",
            prompt_event_id="test_prompt_event_id",
            prompt_template_chat=prompt_template_chat,
            prompt_params=prompt_params,
            chat_id=chat_id,
        )

        # Asserting that the returned event id is as expected
        assert result == "test_event_id"

        assert mock_post.call_count == 2
        mock_post.assert_called_with(
            "https://app.imaginary.dev/api/event",
            json={
                "apiName": "test_api_name",
                "params": {"param1": "value1", "param2": "value2"},
                "prompt": {},
                "promptEventId": "test_prompt_event_id",
                "modelParameters": {},
                "apiKey": "test_api_key",
                "promptTemplateChat": [{"role": "user", "content": "Hello world"}],
                "chatId": "test_chat_id",
            },
        )


@pytest.mark.asyncio
@patch("aiohttp.ClientSession.post", new_callable=AsyncMock)
@patch("os.environ.get")
async def test_send_event_text(mock_env_get, mock_post, mock_env):
    # Mocking the environment variable for URL
    mock_env_get.return_value = "https://app.imaginary.dev/api/event"

    # Mocking the response from the aiohttp post call
    mock_response = AsyncMock()
    mock_response.json.return_value = {"id": "test_event_id"}
    mock_post.return_value = mock_response

    # Creating a ClientSession object
    async with aiohttp.ClientSession() as session:
        # Testing the send_event function with required parameters
        api_key = "test_api_key"
        prompt_template_text = "test_prompt_template_text"
        prompt_params = {"param1": "value1", "param2": "value2"}
        chat_id = "test_chat_id"
        result = await send_event(
            session,
            api_key,
            api_name="test_api_name",
            prompt_event_id="test_prompt_event_id",
        )
        prompt_params = {"param1": "value1", "param2": "value2"}
        chat_id = "test_chat_id"
        result = await send_event(
            session,
            api_key,
            api_name="test_api_name",
            prompt_event_id="test_prompt_event_id",
            prompt_template_text=prompt_template_text,
            prompt_params=prompt_params,
            chat_id=chat_id,
        )

        # Asserting that the returned event id is as expected
        assert result == "test_event_id"

        assert mock_post.call_count == 2
        mock_post.assert_called_with(
            "https://app.imaginary.dev/api/event",
            json={
                "apiName": "test_api_name",
                "params": {"param1": "value1", "param2": "value2"},
                "prompt": {},
                "promptEventId": "test_prompt_event_id",
                "modelParameters": {},
                "apiKey": "test_api_key",
                "promptTemplateText": "test_prompt_template_text",
                "chatId": "test_chat_id",
            },
        )



#!/usr/bin/env python

"""The setup script."""

from setuptools import find_packages, setup

with open("README.md") as readme_file:
    readme = readme_file.read()

with open("HISTORY.md") as history_file:
    history = history_file.read()

requirements = []

test_requirements = [
    "pytest>=3",
]
from im_openai import __version__

setup(
    author="Alec Flett",
    author_email="alec@thegp.com",
    python_requires=">=3.6",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    description="Wrapper library for openai to send events to the Imaginary Programming monitor",
    install_requires=requirements,
    license="MIT license",
    long_description=readme + "\n\n" + history,
    long_description_content_type="text/markdown",
    include_package_data=True,
    keywords="im_openai",
    name="im_openai",
    packages=find_packages(include=["im_openai", "im_openai.*"]),
    test_suite="tests",
    tests_require=test_requirements,
    url="https://github.com/alecf/im_openai",
    version=__version__,
    zip_safe=False,
)
